
#ifndef _NAE_CFG_H_
#define _NAE_CFG_H_
#include <stdio.h>
#include <stdint.h>
#include <assert.h>

#define NUM_NAE_COMPLX 5
#define NUM_GMAC_PORTS_PER_CPLX 4
#define NUM_XAUI_PORTS_PER_CPLX 1

#define NUM_SGMII_PORTS 4 /* For now */
#define NUM_XAUI_PORTS 4
#define NUM_IL_PORTS 2
#define CACHE_LINE_SIZE 64

#define NAE_TX_VC_BASE 476
#define NAE_RX_VC_BASE 1000

enum NAE_MODES {
	INVALID_MODE,
	SGMII_MODE,
	XAUI_MODE,
	INTERLAKEN_MODE
};

/* Common */
extern FILE* cfg_file;
#define CFG_PRINT(args...) fprintf(cfg_file, args)
#define CFG_REG(index, val) CFG_PRINT("\t\t\t\t\t0x%x 0x%08x\n",(index), (val))

extern int total_num_ports;
extern int cur_mode;
extern int cur_complex;
extern int frequency;

extern int get_num_ports();
extern int get_hw_port_id(int intf);

/* FDT API */
extern int get_fdt_prop(const char* node_path, const char *prop, void *prop_buf, int type);

enum {
	PROP_STR,
	PROP_INT
};
extern int prop_len;

#define GET_FPROP(path, prop, prop_buf)					\
	if (get_fdt_prop(path, prop, prop_buf, PROP_INT) != 0) {	\
		printf("Failed to parse %s! Aborting !\n", prop);	\
		exit(1);						\
	}

#define GET_FPROP_STR(path, prop, prop_buf)				\
	if (get_fdt_prop(path, prop, prop_buf, PROP_STR) != 0) {	\
		printf("Failed to parse %s! Aborting !\n", prop);	\
		exit(1);						\
	}


/* NAE */
extern void parse_common_nae_regs(void);
extern void parse_nae_regs(char *complex_str);
extern void gen_nae_cfg(int intf);
extern void compute_nae_config();
extern void nae_reg_cleanup();
extern void gen_global_nae_cfg();

/* GMAC */
extern void parse_gmac_regs(char *complex_str);
extern void gen_gmac_cfg(int intf);
extern int  get_num_gmac_regs(void);

/* XAUI */
extern void parse_xaui_regs(char *complex_str, int num_regs);
extern void gen_xaui_cfg(int intf);

/* INTERLAKEN */
extern void parse_il_regs(char *complex_str, int num_regs);
extern void gen_il_cfg(int intf);

/* POE */
extern void parse_poe_regs();
extern void gen_poe_config();
extern void compute_poe_config();
extern void gmac_reg_cleanup();


#endif /*  _NAE_CFG_H_ */

