#include <stdlib.h>
#include <string.h>
#include "nae-cfg.h"

enum {
	POE_PCI_CFG_REG = 0,
	POE_PCI_MEM_REG = 1,
};
#define CFG_PCIMEM_REG(index, val) CFG_PRINT("\t\t\t\t\t%d 0x%x 0x%08x\n", POE_PCI_MEM_REG, (index), (val))
#define CFG_PCICFG_REG(index, val) CFG_PRINT("\t\t\t\t\t%d 0x%x 0x%08x\n", POE_PCI_CFG_REG, (index), (val))

#define POE_DV_NODE "/poe/distribution_vectors"
#define POE_EMS_NODE "/poe/ext_msg_storage"

#define NUM_DIST_VEC 16
#define NUM_WORDS_PER_DV 16
#define MAX_DV_TBL_ENTRIES (NUM_DIST_VEC * NUM_WORDS_PER_DV)
#define POE_DIST_THRESHOLD_0 0x200
#define POE_DIST_ENABLE 0x205
#define POE_DIST_VEC0_512 0x100
#define POE_DIST_THRESHOLD_VAL 0xa
#define POE_MAX_LOCAL_MSGS (6 << 10) // 6K
#define POE_TX_TIMER     0x214
#define POE_FBP_SP       0xb8
#define POE_FBP_SP_EN    0xb9
#define POE_LOC_ALLOC_EN 0xba
#define POE_EXT_ALLOC_EN 0xbb
#define POE_LOCAL_FBP_BASE 0x400

#define EXT_FBP_START_ADDR       0x1800

enum POE_MODE {
	POE_BYPASS = 0,
	POE_ENQUE = 1,
	POE_SERIAL = 2,
};
static unsigned int dvtbl[MAX_DV_TBL_ENTRIES];
static int num_regs;

static char mode_str[20];

void parse_poe_regs()
{
	int i;
	char str[50];
	unsigned char *buf = (unsigned char *) dvtbl;

	/* Mode */
	GET_FPROP_STR("/poe", "mode", mode_str);

	/* Distribution Vectors */
	for (i=0; i < NUM_DIST_VEC; ++i) {
		
		sprintf(str, "dv%d", i);
		GET_FPROP(POE_DV_NODE, str, buf);
		assert(prop_len == NUM_WORDS_PER_DV);

		buf += (NUM_WORDS_PER_DV * 4);
	}
#if 0
	for (i=0; i < NUM_DIST_VEC; ++i) {
		for (j=0; j < NUM_WORDS_PER_DV; ++j) {
			printf("%d ", dvtbl[(i * NUM_WORDS_PER_DV)  + NUM_WORDS_PER_DV - j - 1]);
		}
		printf("\n");
	}
#endif
}

void gen_poe_config()
{
	int i, j;
	int reg_id = POE_DIST_VEC0_512;
	uint32_t val;

	CFG_PRINT("\t\t\tpoe {\n");
	CFG_PRINT("\t\t\t\tmode = \"%s\";\n", mode_str);
	CFG_PRINT("\t\t\t\tregs {\n");
	CFG_PRINT("\t\t\t\t\tnum-regs = <%d>;\n", num_regs);
	CFG_PRINT("\t\t\t\t\tregs = <\n");
	/* Disable  DIST VEC logic */
	CFG_PCICFG_REG(POE_DIST_ENABLE, 0); 

	for (i=0; i < NUM_DIST_VEC; ++i) {
		for (j=0; j < NUM_WORDS_PER_DV; ++j) {

			val = dvtbl[(i * NUM_WORDS_PER_DV)  + NUM_WORDS_PER_DV - j - 1];
			CFG_PCIMEM_REG(reg_id, val);
			reg_id += 1;
		}
	}
	/* Set the threshold */
	for (i=0; i< 5; ++i) {
		CFG_PCICFG_REG((i + POE_DIST_THRESHOLD_0), POE_DIST_THRESHOLD_VAL);
	}

	/* Enable DIST VEC logic */
	CFG_PCICFG_REG(POE_DIST_ENABLE, 1); 

	/* Local Message Storage: Always enable */
	CFG_PCICFG_REG(POE_LOC_ALLOC_EN, 1);

	/* External Message Storage */
	if (strcmp(mode_str, "bypass") != 0) { /* Non Bypass modes */
		CFG_PCICFG_REG(POE_FBP_SP_EN, 0x1); /* FBP stack pointer enable */

		/* Enable External Buffer allocation */
		CFG_PCICFG_REG(POE_EXT_ALLOC_EN, 1);
	}

	/* POE Tx Timer */
	CFG_PCICFG_REG(POE_TX_TIMER, 0x7);


	CFG_PRINT("\t\t\t\t\t>;\n");
	CFG_PRINT("\t\t\t\t};\n");
	CFG_PRINT("\t\t\t};\n");
}

void compute_poe_config()
{
	/* Currently only Distribution vectors are parsed */
	num_regs = MAX_DV_TBL_ENTRIES;

	/* To Account for extra reg writes
	 * 1. disable DIST_VEC_ENABLE bit (1)
	 * write all the dist vectors
	 * 2. Write threshold vectors (5)
	 * 3. enable DIST_VEC_ENABLE bit (1)
	 */ 
	num_regs += 7; 
	num_regs += 4; /* FBP_SP_EN, TX_TIMER, LOC_ALLOC, EXT_ALLOC */
}
