/***********************************************************************
 * Copyright 2003-2010 Netlogic Microsystems (??Netlogic??). All rights
 * reserved.
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in
 * the documentation and/or other materials provided with the
 * distribution.
 * THIS SOFTWARE IS PROVIDED BY Netlogic Microsystems ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL NETLOGIC OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 * *****************************#NETL_2#********************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#undef	UCORE_DBG
#ifdef  UCORE_DBG
#define ucore_dbg(fmt,args...)     printf(fmt,##args)
#else
#define ucore_dbg(fmt,args...)
#endif

#define ADDRSPACE_SIZE  (1 << 20)
#define MAX_ENTRIES     (ADDRSPACE_SIZE >> 2)

/* local intermediate array */
unsigned int local_array[MAX_ENTRIES];

#define OUTFILE_SUFFIX    ".array.c"
#define INFILE_ELF_SUFFIX ".elf"
#define INFILE_HEX_SUFFIX ".hex"
#define MAX_SUFFIX_LEN    (strlen(OUTFILE_SUFFIX))

int main(int argc, char *argv[]) {

	int ctr=0, i=0;
	unsigned int startaddr, numbytes;
    unsigned int addr_idx = 0, len_idx = 0;
	
	/* We deal with 3 files: -
	 * 1. outfile    : the .c file used by loader
	 * 2. infile_elf : a temporary file to get elf info
	 * 3. infile_hex : the hex file of opcodes
	 */

	const char *arrayname = getenv("ARRAYNAME");
	if (arrayname == NULL) {
		printf("Please set the environment variable ARRAYNAME.\n");
		return -1;
	}
	const int slen = strlen(arrayname) + MAX_SUFFIX_LEN + 1;
	char *outfile    = alloca(slen);
	char *infile_elf = alloca(slen);
	char *infile_hex = alloca(slen);
	sprintf(outfile, "%s"OUTFILE_SUFFIX, arrayname);
	sprintf(infile_elf, "%s"INFILE_ELF_SUFFIX, arrayname);
	sprintf(infile_hex, "%s"INFILE_HEX_SUFFIX, arrayname);

	FILE *fp = fopen(outfile, "w+");
	if (fp == NULL) {
		printf("Unable to open file [%s]!\n", outfile);
		return -1;
	}
	
	FILE *fp_elf = fopen(infile_elf, "r");
	if (fp_elf == NULL) {
		printf("Unable to open file [%s]!\n", infile_elf);
		return -1;
	}
	
	FILE *fp_hex = fopen(infile_hex, "r");
	if (fp_hex == NULL) {
		printf("Unable to open file [%s]!\n", infile_hex);
		return -1;
	}

	/* Read the input hex file
	 */
	while (fscanf(fp_elf, "%x %x", &startaddr, &numbytes) != EOF) {
		if (numbytes & 0x3) {
			printf("numbytes=%d is expected to be multiples of 4!\n", numbytes);
		}
		unsigned int numopcodes = numbytes / 4;
		unsigned int opcode;
		
		ucore_dbg("startaddr:0x%x numopcodes=%d numbytes %d\n", 
				startaddr, numopcodes, numbytes);
        addr_idx = ctr;
		local_array[ctr++] = startaddr;
        len_idx = ctr;
		local_array[ctr++] = numopcodes;

		for (i = 0; i < numopcodes; ++i) {
			fscanf(fp_hex, "%x", &opcode);
			local_array[ctr] = opcode;
			ucore_dbg("opcode:0x%x, ctr:%d\n", local_array[ctr], ctr);
			ctr++;
		}

		// uCore requires an even number of opcodes
		if ((numopcodes % 2) != 0) {
			// insert a dummy nop
			local_array[ctr] = 0;
            numopcodes++;
            local_array[len_idx] = numopcodes;			
            ucore_dbg("inserted opcode:0x%x, ctr:%d numcode %d idx %d\n", local_array[ctr], ctr, numopcodes, len_idx);
			ctr++;
		}
	}

	/* convert this local array to a 
	 * different format to be used
	 * by loader
	 */
	fprintf(fp, "uint32_t %s_array[] = {", arrayname);
	for (i=0; i<ctr; i++) {
		fprintf(fp, "\n\t0x%x", local_array[i]);
		if (ctr < (MAX_ENTRIES - 1))
			fprintf(fp, ",");

	}
	fprintf(fp, "\n};\n");
	fprintf(fp, "#define %s_array_size (%d)\n", arrayname, ctr);

	/* cleanup */
	fclose(fp);
	fclose(fp_elf);
	fclose(fp_hex);

	return 0;
}
